package com.hna.ice.client.core;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * 
 * @author liulibo
 * 
 */
public class CommunicatorHelper {
	private static Log log = LogFactory.getLog(CommunicatorHelper.class);
	private static Object lock = new Object();
	private static CommunicatorHelper _communicatorHelper;
	private String configFile;
	private Ice.InitializationData initData;
	private Ice.Communicator _communicator;
	private Glacier2.RouterPrx _router;
	private Ice.ObjectAdapter _adapter;
	private String _category;
	private long time = 0;

	/**
	 * ʼ
	 * 
	 * @throws java.lang.Exception
	 */
	private void init() throws java.lang.Exception {
		initData = new Ice.InitializationData();
		try {
			initData.properties = Ice.Util.createProperties();
			initData.properties.load(configFile);
			log.info("Communicator Initialized");
		} catch (java.lang.Exception e) {
			log.error(e.getMessage(), e);
			throw e;
		}
		initData.properties.setProperty("Ice.ACM.Client", "0");
		initData.properties.setProperty("Ice.RetryIntervals", "-1");
	}

	/**
	 * ȡCommunicatorHelperʵ
	 * 
	 * @return
	 * @throws java.lang.Exception
	 */
	public static synchronized CommunicatorHelper getInstance() throws java.lang.Exception {
		if (_communicatorHelper == null) {
			_communicatorHelper = new CommunicatorHelper();
		}
		return _communicatorHelper;
	}

	/**
	 * ȡCommunicatorHelperʵ
	 * 
	 * @param configFile
	 * @return
	 * @throws java.lang.Exception
	 */
	public static CommunicatorHelper getInstance(String configFile) throws java.lang.Exception {
		synchronized (lock) {
			if (_communicatorHelper == null) {
				_communicatorHelper = new CommunicatorHelper(configFile);
			}
		}
		return _communicatorHelper;
	}

	private static long getTime() {
		return System.currentTimeMillis();
	}

	private CommunicatorHelper() throws java.lang.Exception {
		this.configFile = "pgs.config";
		init();
		try {
			connect();
		} catch (Exception e) {
			destroy();
			throw e;
		}
	}

	private CommunicatorHelper(String configFile) throws java.lang.Exception {
		this.configFile = configFile;
		init();
		try {
			connect();
		} catch (Exception e) {
			destroy();
			throw e;
		}
	}

	/**
	 * 
	 */
	private void connect() throws java.lang.Exception {
		long time = getTime();
		synchronized (lock) {
			if (time < this.time) {
				return;
			}
			if (_communicatorHelper != null) {
				destroy();
			}
			try {
				_communicator = Ice.Util.initialize(initData);
				_router = Glacier2.RouterPrxHelper.uncheckedCast(_communicator.getDefaultRouter());
			} catch (Throwable e) {
				log.error(e.getMessage());
				throw new Exception(e.getMessage(), e);
			}
			createSession();
			this.time = getTime();
		}
		log.info("Connect with Server");
	}

	/**
	 * Session
	 * 
	 * @return
	 * @throws java.lang.Exception
	 */
	private Glacier2.SessionPrx createSession() throws Exception {
		Glacier2.SessionPrx session = null;
		try {
			session = _router.createSession(initData.properties.getProperty("userId"), initData.properties.getProperty("password"));
			log.info("Create Session");
		} catch (Glacier2.PermissionDeniedException e) {
			log.error("Permission Denied");
			throw new Exception("Permission Denied", e);
		} catch (Glacier2.CannotCreateSessionException e) {
			log.error("Cannot Create Session");
			throw new Exception("Cannot Create Session", e);
		} catch (Ice.ConnectFailedException e) {
			log.error("Connect Failed");
			throw new Exception("Connect Failed", e);
		} catch (Ice.ConnectionLostException e) {
			log.error("Connection Lost");
			throw new Exception("Connection Lost", e);
		} catch (Ice.LocalException e) {
			log.error(e.toString());
			throw new Exception(e.toString(), e);
		} catch (Throwable e) {
			log.error(e.getMessage());
			throw new Exception(e.getMessage(), e);
		}
		return session;
	}

	/**
	 * Իȡ
	 * 
	 * @param property
	 * @return
	 * @throws java.lang.Exception
	 */
	public Ice.ObjectPrx propertyToProxy(String property) throws java.lang.Exception {
		Ice.ObjectPrx base = _communicator.propertyToProxy(property);
		if (base == null) {
			throw new Exception("Not find Proxy");
		}
		if (!ping(base)) {
			base = _communicator.propertyToProxy(property);
		}
		return base;
	}

	/**
	 * Ƿ
	 * 
	 * @param base
	 * @return ӷfalse,򷵻true
	 * @throws java.lang.Exception
	 */
	private boolean ping(Ice.ObjectPrx base) throws java.lang.Exception {
		try {
			base.ice_ping();
			return true;
		} catch (Ice.ConnectFailedException e) {
			log.error("Connect Failed");
			throw new Exception("Connect Failed", e);
		} catch (Ice.ConnectionLostException e) {
			connect();
		} catch (Ice.UnknownLocalException e) {
			log.error("Unknown Local");
			throw new Exception("Unknown Local", e);
		} catch (Ice.RequestFailedException e) {
			log.error("Request Failed");
			throw new Exception("Request Failed", e);
		} catch (Ice.TimeoutException e) {
			log.error("Connection Time out");
			throw new Exception("Connection Time out", e);
		} catch (Ice.CommunicatorDestroyedException e) {
			connect();
		} catch (Ice.SocketException e) {
			connect();
		} catch (Ice.LocalException e) {
			log.error(e.toString());
			throw new Exception(e.toString(), e);
		} catch (Throwable e) {
			log.error(e.getMessage());
			throw new Exception(e.getMessage(), e);
		}
		return false;
	}

	/**
	 * ַȡ
	 * 
	 * @param str
	 * @return
	 * @throws java.lang.Exception
	 */
	public Ice.ObjectPrx stringToProxy(String str) throws java.lang.Exception {
		Ice.ObjectPrx base = _communicator.stringToProxy(str);
		if (base == null) {
			throw new Exception("Not find Proxy");
		}
		if (!ping(base)) {
			base = _communicator.stringToProxy(str);
		}
		return base;
	}

	public String categoryForClient() throws Glacier2.SessionNotExistException {
		if (_router == null) {
			throw new Glacier2.SessionNotExistException();
		}
		return _category;
	}

	public Ice.Identity createCallbackIdentity(String name) throws Glacier2.SessionNotExistException {
		return new Ice.Identity(name, categoryForClient());
	}

	/**
	 * servant
	 * 
	 * @param servant
	 * @return
	 * @throws Glacier2.SessionNotExistException
	 */
	public Ice.ObjectPrx addWithUUID(Ice.Object servant) throws Glacier2.SessionNotExistException {
		return objectAdapter().add(servant, createCallbackIdentity(java.util.UUID.randomUUID().toString()));
	}

	/**
	 * ȡ
	 * 
	 * @return
	 * @throws Glacier2.SessionNotExistException
	 */
	public Ice.ObjectAdapter objectAdapter() throws Glacier2.SessionNotExistException {
		if (_router == null) {
			throw new Glacier2.SessionNotExistException();
		}
		synchronized (this) {
			if (_adapter == null) {
				_adapter = _communicator.createObjectAdapterWithRouter("", _router);
				_adapter.activate();
			}
		}
		return _adapter;
	}

	/**
	 * ͷԴ
	 */
	public void destroy() {
		if (_router != null) {
			try {
				_router.destroySession();
			} catch (Throwable ex) {
			}
			_router = null;
		}
		if (_communicator != null) {
			try {
				_communicator.destroy();
			} catch (Throwable ex) {
			}
			_communicator = null;
		}
		log.info("Communicator destroyed");
	}
}
